import 'dart:io';
import 'package:easy_tui/cli/io/in/terminal_input.dart';
import 'package:easy_tui/cli/io/out/terminal_out.dart';
import 'package:easy_tui/tui/layouts.dart';
import 'package:easy_tui/tui/menu_modifiers.dart';
import 'package:easy_tui/tui/text_dialogue.dart';
import 'package:easy_tui/tui/text_ui.dart';

class TextOptionUI extends TextUI<int> {
  /*
	 * FIELDS
	 */

  static const int BREAK_REPRESENTATION = -2;
  static const String DEFAULT_PREFACE =
      "Enter a number corresponding to an action you would like to perform.";

  /*
	 * CONSTRUCTOR
	 */

  TextOptionUI(
      {List<MenuModifiers> modifiers, Layouts layout = Layouts.VERTICAL})
      : super(modifiers: modifiers, layout: layout);

  /*
	 * GETTERS
	 */

  /// Determines if [value] indicates a `switch` statement should break.
  static bool isBreak(int value) {
    return value == TextOptionUI.BREAK_REPRESENTATION;
  }

  ///	Determines if [input] is valid in context with [this.lines].
  bool _inputIsWithinBounds(int input) {
    return (input <= this.lines.length) && (input > 0);
  }

  /*
	 * OPERATIONS
	 */

  /// Prompt the user for a choice based on discrete options.
  ///
  /// Shows the user a series of predefined options, and loops until they enter
  /// an acceptable input from [this.lines].
  /// If present, [preface] acts as a foreword to the options.
  @override
  int present({String preface = TextOptionUI.DEFAULT_PREFACE, String prepend}) {
    int selectedOptionConverted;
    String selectedOption = "";

    final String writeString = "$preface\n${this.toString()}"
        "${this.promptModifierAdjustments()}"
        "${TextDialogue.PROMPT}";

    while (true) {
      TerminalOut.clear();
      try {
        print(writeString);

        selectedOption = this.usesModifier(MenuModifiers.MAIN_MENU)
            ? TerminalInput.readEvalExit(TextUI.EXIT)
            : stdin.readLineSync();

        selectedOptionConverted = int.parse(selectedOption);

        if (this._inputIsWithinBounds(selectedOptionConverted)) {
          return selectedOptionConverted;
        } else {
          throw new FormatException();
        }
      } on FormatException {
        if (!this.usesModifier(MenuModifiers.NO_EXIT) &&
            (selectedOption == TextUI.EXIT)) {
          return BREAK_REPRESENTATION;
        } else {
          print(
              "Please enter a valid number between 1 and ${this.lines.length}");
          TerminalOut.enterToClear();
        }
      }
    }
  }

  /*
	 * to.../from...
	 */

  @override
  String toString([String prepend]) {
//		Much funnier in java...
    final StringBuffer bob = new StringBuffer();

    final String spaceChar =
        (this.layout == Layouts.VERTICAL) ? "\n" : "      ";

    for (final String str in this.toStringIterative(prepend: prepend)) {
      bob..write(str)..write(spaceChar);
    }

    return bob.toString();
  }
}
